void LCD_Write(int c){
	// [RS][RW][D7][D6][D5][D4][D3][D2][D1][D0]
	LCD_RS  = ((c >> 9) & 1) ^ 1;
	LCD_RW  = ((c >> 8) & 1) ^ 0;
	
	LCD_DB7 = (c >> 7) & 1;
	LCD_DB6 = (c >> 6) & 1;
	LCD_DB5 = (c >> 5) & 1;
	LCD_DB4 = (c >> 4) & 1;
	#if LCD_DL == 0	// 4-bit
		LCD_EN=1;
		__delay_us(LCD_CK_DEL);
		LCD_EN=0;
	#endif
	LCD_DB3 = (c >> 3) & 1;
	LCD_DB2 = (c >> 2) & 1;
	LCD_DB1 = (c >> 1) & 1;
	LCD_DB0 = (c >> 0) & 1;
	
	LCD_EN=1;
	__delay_us(LCD_CK_DEL);
	LCD_EN=0;
	
	__delay_ms(2);
}

void LCD_Initialize(){
	LCD_RS=0;
	LCD_RW=0;
	LCD_EN=0;
	
	__delay_ms(LCD_PWR_DELAY);
	
	// 4-BIT ONLY
	#if LCD_DL == 0
		LCD_DB7 = 0;
		LCD_DB6 = 0;
		LCD_DB5 = 1;
		LCD_DB4 = 1;
		LCD_EN=1;
		__delay_us(LCD_CK_DEL);
		LCD_EN=0;
		
		__delay_ms(5);
		LCD_EN=1;
		__delay_us(LCD_CK_DEL);
		LCD_EN=0;
		
		LCD_DB7 = 0;
		LCD_DB6 = 0;
		LCD_DB5 = 1;
		LCD_DB4 = 0;
		LCD_EN=1;
		__delay_us(LCD_CK_DEL);
		LCD_EN=0;
	#endif

	LCD_Write(LCD_FUNCTION_SET(LCD_DL,LCD_N,LCD_F));
	LCD_Write(LCD_CD_SHIFT(LCD_SC,LCD_RL));
	LCD_Write(LCD_DISPLAY_CONTROL(LCD_D,LCD_C,LCD_B));
	LCD_Write(LCD_ENTRY_MODE_SET(LCD_S,LCD_ID));
	
	LCD_Write(LCD_CLEAR);
	__delay_ms(2);
}

void LCD_WriteString(const char * s){
	while(*s){
		if (*s != '\n') LCD_Write(*s);
		else LCD_Write(LCD_GOTO(0x40));
		
		s++;
	}
}

void LCD_WriteNumber(unsigned long n){
	if (n==0) LCD_Write('0');
	else {
		static char s[11];
		char p=0;
		while (n){
			s[p] = (n%10) + '0';
			n /= 10;
			p++;
		}
		
		while (p--) LCD_Write(s[p]);
	}
}