/******************************************************************************
*         HITACHI HD44780 DISPLAY CONTROLLER C DRIVER FOR PIC16 MCUs          *
*                                                                             *
* AUTHOR:   COSTACHESCU CRISTINEL (a.k.a. T3STY)                              *
* VERSION:  3.0                                                               *
*           [CHANGELOG FOR VERSION 3.0]                                       *
*           # LCD_Write function rewritten for less code overhead             *
*             --> All driver functions have been rewritten too to accord with *
*                 the new LCD_Write function.                                 *
*                                                                             *
*           [CHANGELOG FOR VERSION 2.1]                                       *
*           + Added compiler directives for 4-bit initialization              *
*           + Added compiler directives for 8-bit DATA bus definitions        *
*                                                                             *
* CONTACT:  Contact me via Private Messages on the edaboard forum:            *
*           http://www.edaboard.com/member467136.html                         *
*           or via email at:                                                  *
*           T3STY@live.com                                                    *
*                                                                             *
* LICENSE: GPL 3.0 - Please read License.txt provided along with this package *
*          or read it online at:                                              *
*          http://www.gnu.org/licenses/gpl.txt                                *
*                                                                             *
* DISCLAIMER: Please read Disclaimer.txt provided along with this package     *
*             Also, this file (HD44780.h) is OVER-COMMENTED. Although the     *
*             HD44780.c file is undercommented (self-explainatory in most     *
*             cases) 'I' made the effort of explaining here how to correctly  *
*             use this driver; make sure 'YOU' read them comments for your    *
*             own good (and no headaches - if any).                           *
*                                                                             *
* KNOWN BUGS:                                                                 *
*   # LCD_WriteNumber() cannot write signed numbers                           *
*   # This driver does not have any function for reading the BusyFlag         *
*     >> I may never add one, feel free to make your own if you're able to    *
*                                                                             *
* NOTES:                                                                      *
*   # This driver relies on HiTech PICC Compiler's delay functions:           *
*     >>  __delay_us()  for micro-seconds                                     *
*     >>  __delay_ms()  for milli-seconds                                     *
*     if you're willing to use any other compiler make sure you find an       *
*     equivalent replacement for these functions                              *
*                                                                             *
******************************************************************************/

/******************************************************************************
*                             LCD Base settings                               *
******************************************************************************/
// FUNCTION SET
#define LCD_DL		0	// 0: 4-bit DTM		| 1: 8-bit Data Transfer Mode
#define LCD_N		1	// 0: 1 line		| 1: 2 lines
#define LCD_F		0	// 0: 5x8 px font	| 1: 5x10 px font

// ENTRY MODE SET
#define LCD_S		0	// 0: SHIFT	OFF		| 1: SHIFT ON
#define LCD_ID		0	// 0: INCREMENT		| 1: DECREMENT

// DISPLAY ON/OFF CONTROL
#define LCD_B		0	// 0: CUR_BLINK OFF	| 1: CUR_BLINK ON
#define LCD_C		0	// 0: CURSOR OFF	| 1: CURSOR ON
#define LCD_D		1	// 0: DISPLAY OFF	| 1: DISPLAY ON

// CURSOR OR DISPLAY SHIFT
#define LCD_SC		0	// 0: SHIFT CURSOR	| 1: SHIFT DISPLAY
#define LCD_RL		1	// 0: SHIFT LEFT	| 1: SHIFT RIGHT

/******************************************************************************
*                      PIC -> LCD Interface Control pins                      *
******************************************************************************/

// ADDRESS BUS
#define LCD_RS  	RB1	// Register Select

// CONTROL BUS
#define LCD_RW  	RB7	// Read/Write
#define LCD_EN  	RB0	// Enable
#define LCD_CK_DEL	 50	// us

// DATA BUS
#define LCD_DB7		RC3	// DB7
#define LCD_DB6		RC2	// DB6
#define LCD_DB5		RC1	// DB5
#define LCD_DB4		RC0	// DB4
#if LCD_DL == 0  // 4 BIT ONLY
	#define LCD_DB3 LCD_DB7
	#define LCD_DB2 LCD_DB6
	#define LCD_DB1 LCD_DB5
	#define LCD_DB0 LCD_DB4
#else            // 8 BIT ONLY
	#define LCD_DB3 RC4	// DB3
	#define LCD_DB2 RC5	// DB2
	#define LCD_DB1 RC6	// DB1
	#define LCD_DB0 RC7	// DB0
#endif

#define LCD_PWR_DELAY 20 /* ms */

#define LCD_CW		0x0200
#define LCD_CR		0x0300
#define LCD_DW		0x0000
#define LCD_DR		0x0100

// SET DDRAM ADDRESS
// | 1  A  A  A  A  A  A  A |
#define LCD_GOTO(x)	x+0x0280

// FUNCTION SET
// | 0  0  1  DL N  F  0  0 |
#define LCD_FUNCTION_SET(DL,N,F)	((((DL<<4)&0x10)+((N<<3)&0x08)+((F<<2)&0x04))|0x0220)

// CURSOR OR DISPLAY SHIFT
// | 0  0  0  1  SC RL 0  0 |
#define LCD_CD_SHIFT(SC,RL)			((((SC<<3)&0x08)+((RL<<3)&0x04))|0x0210)

// DISPLAY ON/OFF CONTROL
// | 0  0  0  0  1  D  C  B |
#define LCD_DISPLAY_CONTROL(D,C,B)	((((D<<2)&0x04)+((C<<1)&0x02)+(B&0x01))|0x0208)

// ENTRY MODE SET
// | 0  0  0  0  0  1  ID S |
#define LCD_ENTRY_MODE_SET(ID,S)	((((ID<<1)&0x02)+(S&0x01))|0x0204)

// MOVE CURSOR HOME
// | 0  0  0  0  0  0  1  0 |
#define LCD_HOME	0x0202

// CLEAR DISPLAY
// | 0  0  0  0  0  0  0  1 |
#define LCD_CLEAR	0x0201

void LCD_Write(int);
void LCD_Initialize();
void LCD_WriteString(const char * s);
void LCD_WriteNumber(unsigned long);
