#define _XTAL_FREQ 4000000
#include <htc.h>

__CONFIG(FOSC_XT & WDTE_OFF & PWRTE_ON & CP_OFF & BOREN_OFF & LVP_OFF & CPD_OFF & WRT_ON);
//__EEPROM_DATA(1,30,5,0,20,0,0,0);
__EEPROM_DATA(0,5,5,0,20,0,0,0);

/***** DRIVER LCD ****/
#include "HD44780.h"
#include "HD44780.c"

/******* INPUT *******/
#define INPUT_DEC	RA0
#define INPUT_INC	RA1
#define INPUT_SEL	RA2
#define INPUT_PAN	RA4
#define INPUT_RIT	200  /* ms */

/******* OUTPUT ******/
#define LED_STATO	RB2	// LED di stato
#define COM_UV		RB3	// Comando LED UV

/** INDIRIZZI EEPROM **/
// TIMER ESPOSIZIONE
#define TIMER_MIN	0x00
#define TIMER_SEC	0x01
// TIMER CALIBRAZIONE
#define CALIB_MIN	0x02
#define CALIB_SEC	0x03
#define CALIB_PAS	0x04

/* IMPOSTAZIONI GENERALI */
#define TIMER_ON	0xA0
#define TIMER_OFF	0x00
#define PANNELLO_AP	0x00
#define PANNELLO_CH	0x01

const char * M_Pri[] = {
	"AVVIA ESPOSIZ.\n",
	"AVVIA CALIBRA.\n",
	"IMPOSTAZIONI   "
};

const char * M_Imp[] = {
	"IMPOSTA CALIBR.\n",
	"IMPOSTA TIMER"
};

// Contatore di clock del TMR0
char TMR0_CK=0;
// Indicatore asincrono di aggiornamento timer
bit AGG_TIMER;

char VisualizzaMenu(char [], char);
void ImpostaCalibrazione();
void ImpostaTimer(char, char);
void Esposizione();

void main(){
	// PORTA: tutto digitale
	ADCON1=0x06;
	
	TRISA=0xFF; // PORTA: tutti input
	TRISB=0x00;
	TRISC=0x00;
	PORTA=0x00;
	PORTB=0x00;
	PORTC=0x00;
	
	// TMR0 Prescaler (PS2|PS1|PS0): 1|0|0 -> 1:32
	OPTION_REG=0x84;
	
	// Aggiornamento timer inizialmente disabilitato
	// Verrà abilitato nella routine T0ISR
	AGG_TIMER=0;
	
	LCD_Initialize();
	
	char COM;
	
	while(1){
		COM = VisualizzaMenu(M_Pri, 3);
		LCD_Write(LCD_CLEAR);
		switch (COM){
			// Avvia esposizione
			case 0: {
				Esposizione();
				break;
			}
			// Avvia calibrazione
			case 1:{
				LCD_WriteString("CALIBRAZIONE");
				break;
			}
			// Impostazioni
			case 2:{
				COM = VisualizzaMenu(M_Imp, 2);
				LCD_Write(LCD_CLEAR);
				switch (COM){
					// Imposta calibrazione
					case 0: {
						ImpostaCalibrazione();
						break;
					}
					
					// Imposta timer
					case 1: {
						ImpostaTimer(TIMER_MIN, TIMER_SEC);
						break;
					}
				}
				break;
			}
		}
		__delay_ms(1000);
	}
}

void interrupt T0ISR(){
	if (T0IF){
		TMR0_CK++;
		if (TMR0_CK == 124){
			TMR0_CK=0;
			AGG_TIMER=1;
		}
		
		T0IF=0;
	}
}

char VisualizzaMenu(char menu[], char righe){
	char sel=0;
	static bit Agg_LCD;
	Agg_LCD=1;
	
	do{
		// se necessario...
		if (Agg_LCD){
			// ...aggiorna il display
			Agg_LCD=0;
			
			LCD_Write(LCD_CLEAR);
			
			if (sel < (righe-1)){
				LCD_Write('>');
				LCD_WriteString(menu[sel]);
				LCD_Write(' ');
				LCD_WriteString(menu[sel+1]);
			} else {
				LCD_Write(' ');
				LCD_WriteString(menu[sel-1]);
				LCD_Write('>');
				LCD_WriteString(menu[sel]);
			}
		}
		
		// Attendi tra un input e il prossimo
		__delay_ms(INPUT_RIT);
		
		// Gestisci gli input
		if ((INPUT_DEC == 1) && (sel > 0)){
			sel--;
			Agg_LCD=1;
		} else if ((INPUT_INC == 1) && (sel < (righe-1))){
			sel++;
			Agg_LCD=1;
		}
	} while (!INPUT_SEL); // è stata selezionata una voce: terminare la procedura
	
	return sel; // riportare la voce selezionata
}

void ImpostaCalibrazione(){
	// Modifica del tempo totale di calibrazione
	LCD_Write(LCD_CLEAR);
	LCD_Write(LCD_DISPLAY_CONTROL(LCD_D,LCD_C,1));
	LCD_WriteString("IMPOSTARE IL\nTEMPO TOTALE  >>");
	LCD_Write(LCD_GOTO(0x4F));
	__delay_ms(INPUT_RIT);
	while(!INPUT_SEL);
	ImpostaTimer(CALIB_MIN, CALIB_SEC);
	
	// Modifica del passo di calibrazione
	LCD_Write(LCD_DISPLAY_CONTROL(LCD_D,LCD_C,1));
	LCD_Write(LCD_CLEAR);
	LCD_WriteString("IMPOSTARE IL\nPASSO         >>");
	LCD_Write(LCD_GOTO(0x4F));
	__delay_ms(INPUT_RIT);
	while(!INPUT_SEL);
	char PAS = eeprom_read(CALIB_PAS);
	
	LCD_Write(LCD_CLEAR);
	LCD_WriteString("     PASSO\n    secondi");
	LCD_Write(LCD_GOTO(0x41));
	
	// Abilitazione del cursore lampeggiante
	do {
		__delay_ms(INPUT_RIT);
		if (INPUT_DEC && (PAS > 10))
			PAS-=5;
		else if (INPUT_INC && (PAS < 60))
			PAS+=5;
		
		LCD_WriteNumber(PAS);
		LCD_Write(LCD_GOTO(0x41));
	} while (!INPUT_SEL);
	
	// Disabilitazione del cursore lampeggiante
	LCD_Write(LCD_DISPLAY_CONTROL(LCD_D,LCD_C,0));
	
	// Salvataggio del passo di calibrazione
	eeprom_write(CALIB_PAS, PAS);
}

void ImpostaTimer(char aMIN, char aSEC){
	// Lettura del tempo attuale dalla EEPROM
	char MIN = eeprom_read(aMIN);
	char SEC = eeprom_read(aSEC);
	
	do {
		// Schermata iniziale
		// | |M|I|N|U|T|I| |S|E|C|O|N|D|I| |
		// | | | |M|M| | | | | | |S|S| | | |
		LCD_Write(LCD_CLEAR);
		LCD_WriteString(" MINUTI SECONDI");
		
		LCD_Write(LCD_GOTO(0x43));
		if (MIN < 10) LCD_Write('0');
			LCD_WriteNumber(MIN);
		
		LCD_Write(LCD_GOTO(0x4B));
		if (SEC < 10) LCD_Write('0');
			LCD_WriteNumber(SEC);

		// Abilitazione del cursore lampeggiante
		LCD_Write(LCD_DISPLAY_CONTROL(LCD_D,LCD_C,1));
		
		// Modifica dei minuti
		LCD_Write(LCD_GOTO(0x43));
		do {
			__delay_ms(INPUT_RIT);
			if (INPUT_DEC && (MIN > 0))
				MIN--;
			else if (INPUT_INC && (MIN < 59))
				MIN++;
			
			if (MIN < 10) LCD_Write('0');
			LCD_WriteNumber(MIN);
			LCD_Write(LCD_GOTO(0x43));
		} while (!INPUT_SEL);
		
		// Modifica dei secondi
		LCD_Write(LCD_GOTO(0x4B));
		do {
			__delay_ms(INPUT_RIT);
			if (INPUT_DEC && (SEC > 0))
				SEC-=5;
			else if (INPUT_INC && (SEC < 55))
				SEC+=5;
			
			if (SEC < 10) LCD_Write('0');
			LCD_WriteNumber(SEC);
			LCD_Write(LCD_GOTO(0x4B));
		} while (!INPUT_SEL);
		
		// Disabilitazione del cursore lampeggiante
		LCD_Write(LCD_DISPLAY_CONTROL(LCD_D,LCD_C,0));
		
		// Controllo di validità del tempo
		if ((MIN == 0) && (SEC == 0)){
			LCD_Write(LCD_CLEAR);
			LCD_WriteString("    ERRORE!\nTEMPO NON VALIDO");
			__delay_ms(1500);
		}
	} while ((MIN == 0) && (SEC == 0));
	
	// Memorizzazione del nuovo tempo nella EEPROM 
	eeprom_write(aMIN, MIN);
	eeprom_write(aSEC, SEC);
}

void Esposizione(){
	char MIN = eeprom_read(TIMER_MIN);
	char SEC = eeprom_read(TIMER_SEC);
	
	INTCON = TIMER_ON;
	LCD_Write(LCD_CLEAR);
	LCD_WriteNumber(MIN);
	LCD_Write(':');
	LCD_WriteNumber(SEC);
	do {
		if (AGG_TIMER){
			if (SEC > 0) SEC--;
			else {
				SEC = 59;
				MIN--;
			}
			LCD_Write(LCD_CLEAR);
			LCD_WriteNumber(MIN);
			LCD_Write(':');
			LCD_WriteNumber(SEC);
			
			AGG_TIMER = 0;
		}
	} while (MIN || SEC);
	
	INTCON = TIMER_OFF;
	
	LCD_Write(LCD_CLEAR);
	LCD_WriteString("Esposizione\ncompletata!"); /* HERE IS WHERE THE ISSUE HAPPENS */
}
